#!/usr/bin/python
# -*- coding: utf-8 -*-
#Copyright 2008 Calculate Pack, http://www.calculate-linux.ru
# 
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
# 
#      http://www.apache.org/licenses/LICENSE-2.0
# 
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

import os
import sys
import serial
import threading
import time
import re
import string

BELL = 1
CCITT = 0
LOW = 1
MID = 2
HIGH = 3
OFF = 0
RINGANDCONNECT = 1
ALWAYS = 2
AFTERRINGBEFORECONNECT = 3
NUMBER = 0
WORD = 1

MESSAGE_SHORT = 0
MESSAGE_FULL = 1
WAIT_DIALTONE = 2
NOT_WAIT_DIALTONE = 0
BUSY_DETECT = 4
BUSY_NOTDETECT = 0

DISCONNECT_ON_BREAK = 1
IGNORE_BREAK = 0

NVRAM0 = 0
NVRAM1 = 1
GENERICTEMPLATE = 2
HARDFLOWCONTROL = 3
SOFTFLOWCONTROL = 4

NUL = chr(0x00)
ETX = chr(0x03)
NL =  chr(0x0a)
CR =  chr(0x0d)
DLE = chr(0x10)
XON = chr(0x11)
XOFF =chr(0x13)
DC4 = chr(0x14)
CAN = chr(0x18)
FS =  chr(0x1c)
SUB = chr(0x20)
ENDFAX = chr(0x2e)

# добавлена для удобства обозначения команд без префикса AT
NOT_AT =""

# ошибка в ходе инициализации модема
class ModemError(Exception):
    pass

# ошибка, поднимаемая в следствии прошествия времени таймаута
class TimeoutError(Exception):
    pass

# модем в ответ на ввод какой-либо команды ответил ERROR
class ErrorAnswer(Exception):
    pass

# сообщение от модема, что на другом конце линии занято или повесили трубку
class BusyAnswer(Exception):
    pass

# базовые функции работы с модемом
class Modem:
    def addevent(self, fn, data ):
        '''Внутренняя функция постановки события в очередь

        fn -- объект-функция с параметром или без
        data -- передаваемый параметр в функцию
            
        '''
        self._lock.acquire()
        self._mess.insert(0,fn)
        self._mess.insert(0,data)
        self._lock.release()

    def reader(self):
        '''Внутренняя функция циклического чтения из входного потока модема.
        '''
        # флаг - предыдущий прочитанный символ DLE
        wasDLE = False
        # флаг - предыдущий прочитанный символ CR
        wasCR = False
        # флаг - началась передача голосовых данных
        wasVoiceData = False
        # флаг - получена команда VRX
        VRXreceive = False
#        flog = open( 'advmodem.log', 'a'  )
#        flog.write( time.ctime() + "\n" )
        while self._bWork:
            # читаем очередной символ с порта
            # если в порту есть символ
            if self._serial.inWaiting()>0:
                # закрываем замок работы с портом и читаем буффер, освобождаем
                self._writelock.acquire()
                dataall = self._serial.read(self._serial.inWaiting() )
                self._writelock.release()
                # перебираем все прочитанные символы
                for data in dataall:
#                    flog.write( data.__repr__() )
#                    flog.flush()
                    # если предыдущий символ ДЛЕ
                    if wasDLE:
                        # сбрасываем флаг дле символа
                        wasDLE = False
                        # если символ ДЛЕ
                        if data == DLE:
                            # в буффер помещается ДЛЕ символ
                            self._buf+= DLE
                        # если ЕТХ (конец записи)
                        elif data == ETX:
                            # помещаем в очередь событие получения ЕТХ
                            # событию передаем весь записаный буффер
                            self.addevent(self._etxreceive, self._buf )
                            # чистим буффер
                            self._buf = ''
#                            flog.write(NL+"End receiving data:"+NL) 
#                            flog.flush()
                            # сбрасываем флаг ожидания голосовых данных
                            wasVoiceData = False
                        elif data == SUB:
                            self._buf+= DLE + DLE
                        else:
                            # помещаем в очередь событий сигнал ДЛЕ
                            self.addevent(self._dlereceive, data )
                    # не дле символы
                    else:
                        # если символ - ДЛЕ взводим флаг
                        if data == DLE:
                            wasDLE = True
                        # если взведен флаг голосоывах данных
                        elif wasVoiceData:
                            # добавляем символ к буфферу
                            # (избегаем отлова возврата каретки)
                            self._buf+= data
                        # если предыдущий перевод каретки
                        elif wasCR:
                            wasCR = False
                            # а текущий новая строка
                            if data == NL:
                               # добавляем событие новая строка ответа от модем
                               self.addevent(self._answer, self._buf )
                               # если взведен флаг получения команды +VRX
                               # и текущее сообщение CONNECT
                               if VRXreceive and self._buf == 'CONNECT':
                                   # переключем в режим получения голосовых
                                   wasVoiceData = True
                                   # сбрасываем флаг получен (+VRX)
                                   VRXreceive = False
                               # если получена VRX команда, взводим флаг
                               if len(self._buf)>5 \
                                          and (self._buf[-5:] == "+VRX\r" \
                                     or self._buf[-5:] == "+FDR\r"):
#                                   flog.write(NL+"Begin receiving data:"+NL) 
#                                   flog.flush()
                                   VRXreceive = True
                               # очищаем буффер
                               self._buf =''
                            # если символ не новая строка
                            else:
                               # добавляем предыдущий символ возврат каретки
                               self._buf+= CR
                               # если символ возврат кар, взводим флаг
                               if data == CR:
                                  wasCR = True
                               else:
                                  # добавляем символ в магазин
                                  self._buf+= data
                        # если предыдущий обычный символ
                        else:
                            # если символ возврата каретки
                            if data == CR:
                                wasCR = True
                            else:
                                # помещаем символ в магазин
                                self._buf+= data
            # если в порту нет символа, засыпаем
            else:
                time.sleep(0.1)
        # закрываем порт
        self._serial.close()
        if self._verbose:
            print "Port is closing"
            sys.stdout.flush()

    def __init__(self, portname = '/dev/ttyACM0', baudrate = 57600):
        '''Открывает порт модема и запускает поток на чтение данных

        portname -- имя порта модема (/dev/ttyACM0)
        baudrate -- скорость подключения к модему

        '''
        # проверка связанная с множественным наследованием
        # если modem наследуется voicemodem и faxmodem, то
        # чтобы при инициализации zyxelomni (voice,fax) 
        # базовый класс не инициализировался дважды
        try:
            temp = self._mess
        except AttributeError:
            # очередь сообщений
            self._mess = []
            # флаг работы циклического чтения данных
            self._bWork = True
            try:
                self._serial = serial.Serial( portname, baudrate, \
                              rtscts=0, xonxoff=0, timeout=1 )
                self._serial.write( '\rat\r' )
                self._serial.read(30)
            except:
                raise ModemError, "Не могу открыть порт."
            self._lock = threading.Lock() # замок на события
            self._waitanswer = True  # флаг ожидания ответа
            self._buf = ''           # буффер для накопления данных с порта
            self._what = []          # список допустимых ответов от модема
            self._verbose = True     # режим отображения хода работы
            self._timeout = 15       # таймаут ожидания ответа от модема
            self._answers = []       # ответы от модема на команду
            self._raiseBusy = True   # исключения на Бизи
            self._uptubeonring = False # поднять трубку на звонок
            self._r = threading.Thread( target=self.reader )
            self._r.start() # поток чтения данных с модема
            self._writelock = threading.Lock() # замок работы с портом
            self._critical_write = False # 

    def _dlereceive(self, data ):
        '''Заглушка на получение ДЛЕ'''
        pass

    def _etxreceive(self,data ):
        '''Заглушка на получение ETX'''
        pass

    def test(self):
        '''Тест команда'''
        self.at()

    def close(self):
        '''завершить поток и закрыть порт'''
        self._bWork = False

    def OnRing(self, t):
        if self._uptubeonring:
            if self._verbose:
                print "ATA"
                sys.stdout.flush()
            self.at( "A", ["OK","VCON","+FCO\r"] )

    def OnAnswer(self,s):
        pass

    def _answer(self,s):
        '''событие получен ответ от модема'''
        if self._verbose:
            print s.__repr__()
            sys.stdout.flush()
        if s == 'RING':
            self.addevent( self.OnRing, '' )
        self.OnAnswer(s)
        # если ответ из допустимых
        if s in self._what:
            self._what = []
            self._answers = self._answers[1:]
        elif s == 'ERROR':
            raise ErrorAnswer()
        elif s == 'BUSY' and self._raiseBusy:
            raise BusyAnswer()
        else:
            self._answers.append(s)

    def _at(self, s ):
        if self._critical_write:
            self._writelock.acquire()
            self._serial.write( s )
            self._writelock.release()
        else:
            self._serial.write( s )

    def at(self, s="", answers=["OK"], dop="at" ):
        self._lock.acquire()
        self._answers = []
        self._mess.insert(0, self._at)
        self._mess.insert(0, dop+s+"\r" )
        self._what = answers
        self._lock.release()

        self.Run()
        c = time.time()
        while len(self._what) > 0:
            if time.time()-c > self._timeout:
                raise TimeoutError()
            self.Run()
            sys.stdout.flush()
            time.sleep(0.02)
        return self._answers

    def reset(self):
       self.at("Z", ["OK"])

    def dial(self,number, timeout=60):
       number = str(number)
       oldtimeout = self._timeout
       self._timeout = timeout
       self.at("d"+number, ["OK","BUSY","VCON"])
       self._timeout = oldtimeout

    def Run(self):
        while len(self._mess) > 1:
            self._lock.acquire()
            fn = self._mess.pop()
            data = self._mess.pop()
            self._lock.release()
            if data == None:
                fn()
            else:
                fn(data)
#        except ErrorAnswer:
#            self._lock.release()
#            raise ErrorAnswer()
#        except BusyAnswer:
#            self._lock.release()
#            raise BusyAnswer()
#        except:
#            self._lock.release()

class FaxSwapTable:
    def __init__(self):
        self.table = ""
        for i in range(0,256):
            self.table+= chr( ((i & 0x01) << 7) | ((i & 0x02) << 5) |
                             ((i & 0x04) << 3) | ((i & 0x08) << 1) |
                             ((i & 0x10) >> 1) | ((i & 0x20) >> 3) |
                             ((i & 0x40) >> 5) | ((i & 0x80) >> 7 ) )
    def __call__(self,i):
        return self.table[ord(i)]

class VoiceModem(Modem):
    def __init__(self, portname= '/dev/ttyACM0', baudrate= 57600):
        Modem.__init__( self, portname, baudrate )
        self._filename = "test.rmd"
        self.stop_playing = False
        self.play_complete = False

    def setVoiceMode(self):
        self.at("+FCLASS=8", ["OK"])

    def setDataMode(self):
        self.at("+FCLASS=0", ["OK"])

    def setInactiveTime(self, num= 100 ):
        '''Установить порог неактивного времения'''
        self.at( "+VIT=%d" % num, ["OK"] )

    def setSilenceDetection(self, sds=12, sdi=70 ):
        '''Set the silence detection sensivity'''
        self.at( "+VSD=%d,%d" % (sds,sdi), ["OK"] )

    def setTransmitGain(self, num):
        '''Уровень передаваемого сигнала, устанавливает чувствительность
        микрофона в режиме SpeakerPhone.'''
        self.at( "+VGT=%d" % num, ["OK"] )

    def setReceiveGain(self, num):
        '''Уровень принимаемого сигнала, устанавливает уровень громкости
        динамика в режиме SpeakerPhone'''
        self.at( "+VGR=%d" % num, ["OK"] )

    def setRingbackAway(self, num):
        '''Время отсутствия сигнала контроля вызова.
        Время отстутствия гудка, после которого модем считает, что трубку
        сняли'''
        self.at( "+VRA=%d" % num, ["OK"] )

    def setRingbackCame(self, num):
        '''Время до появления первого сигнала контроля вызова'''
        self.at( "+VRN=%d" % num, ["OK"] )

    def setVoiceInputOutput(self,num):
        '''Установить устройство ввода/вывода голосовых данных'''
        self.at( "+VLS=%d" % num, ["OK","VCON"] )

    def setSampleMethod(self, method,vrs):
        '''Установить метод сжатия данных (метод сжатия, частота)'''
        self.at( "+VSM=%d,%d" % (method,vrs), ["OK"] )

    def setFlowControl(self, num):
        '''Установить FlowControl (1 - XON/XOFF, 2 - CCITT)'''
        self.at( "+FLO=%d" % num, ["OK"] )

    def getCurMode(self):
        return self.at("+FCLASS?")

    def Busy(self):
        if self._verbose:
            print "BUSY"
            sys.stdout.flush()

    def _dlereceive(self, data ):
        if( data == 'b' ):
            self.Busy()
        elif( data == 'r' ):
            if self._verbose:
                print "RING"
                sys.stdout.flush()
        else:
            if self._verbose:
                print "<DLE>%s</DLE>" % data
                sys.stdout.flush()

    def _etxreceive(self,data ):
        if len(self._filename) > 0:
            f = open( self._filename, "w" )
            s = "RMD1ZyXEL Omni 56K" + NUL*3+chr(4)+ \
                  chr(0x25)+chr(0x80)+chr(0x04)+NUL*7
            f.write(s)
            f.write( data )
            f.close()
            self._filename =""

    def startRecordVoice(self, filename="test.rmd"):
        self._filename = filename
        self.setSampleMethod(4, 9600 )
        self.at( "+VTS=[500,1300,30]" )
        self.at( "+VRX", ["CONNECT"] )

    def stopRecordVoice(self):
        self.at( "w", ["OK"], NOT_AT )

    def startPlayVoice(self):
        self.at("+VTX", "CONNECT")

    def playWavFile(self, filename, baudrate=9600):
        '''Возвращает исключение IOError, если файл нет на диске'''
        f = os.popen2( "cat " + filename + " | wavtopvf | pvftormd" + \
                " ZyXEL_Omni56K 4" )[1]
        f.read(32) # заголовочная информация
        s = f.read()
        f.close()
        fgetc = iter(s)
        self.stop_playing = False
        self.play_complete = False
        bytes_max = baudrate/10
        while not self.stop_playing and not self.play_complete:
            output_buffer = []
            bytes_out = 0
            while bytes_out < bytes_max:
                try:
                    date_byte = fgetc.next()
                except StopIteration:
                    self.play_complete = True
                    break
                output_buffer.append(date_byte)
                if date_byte == DLE:
                    bytes_out+= 2
                    output_buffer.append(DLE)
                else:
                    bytes_out+= 1
            self._at( string.join( output_buffer, "" ) )
            #time.sleep(0.1)
            self.Run()

    def playFile(self, filename, baudrate=9600):
        '''Возвращает исключение IOError, если файл нет на диске'''
        f = open( filename, "r" )
        f.read(32) # заголовочная информация
        s = f.read()
        f.close()
        fgetc = iter(s)
        self.stop_playing = False
        self.play_complete = False
        bytes_max = baudrate/10
        while not self.stop_playing and not self.play_complete:
            output_buffer = []
            bytes_out = 0
            while bytes_out < bytes_max:
                try:
                    date_byte = fgetc.next()
                except StopIteration:
                    self.play_complete = True
                    break
                output_buffer.append(date_byte)
                if date_byte == DLE:
                    bytes_out+= 2
                    output_buffer.append(DLE)
                else:
                    bytes_out+= 1
            self._at( string.join( output_buffer, "" ) )
            #time.sleep(0.1)
            self.Run()

    def stopPlayVoice(self):
        '''Остановить проигрывания файла'''
        self.at( DLE + ETX, ["OK"], NOT_AT)

class FaxData:
    def __init__(self):
        self.vr = -1
        self.br = -1
        self.wd = -1
        self.ln = -1
        self.df = -1
        self.ec = -1
        self.bf = -1
        self.st = -1

class FaxModem(Modem):
    def __init__(self, portname='/dev/ttyACM0', \
                    baudrate=57600, faxname="Calculate Fax"):
        Modem.__init__( self, portname, baudrate )
        self._faxfilename = ''
        self.stop_sending = True
        self.send_complete = True
        self.ft = FaxSwapTable()
        self.faxname = faxname
        self.modemDetected = False
        self.waitFaxSignal = False

    def sendFile(self, fd, baudrate=9600):
        s = fd.read()
        fd.close()
        fgetc = iter(s)
        self.stop_sending = False
        self.send_complete = False
        bytes_max = baudrate/10
        self._at( chr(0)*100 )
        curtime = time.time()
        while not self.stop_sending and not self.send_complete \
                and time.time() - curtime < 240:
            output_buffer = []
            bytes_out = 0
            while bytes_out < bytes_max:
                try:
                    date_byte = self.ft( fgetc.next() )
                except StopIteration:
                    self.send_complete = True
                    break
                output_buffer.append(date_byte)
                if date_byte == DLE:
                    bytes_out+= 2
                    output_buffer.append(DLE)
                else:
                    bytes_out+= 1
            self._at( string.join( output_buffer, "" ) )
            self.Run()

    def WaitFax( self ):
        '''ожидание звонка и прием факса по звонку'''
        self.Init()
        self.setFaxMode()
        self._uptubeonring = True
        while True:
            self.Run()
            time.sleep(0.1)

    def OnFaxConnect(self):
        # если получен FCO и режим поднятой трубки, то добавляем событие прием факса
        if self._uptubeonring:
            self.addevent( self.startReceivingFax, None )

    def setFaxMode(self):
        self.at("&H4+FCLASS=2.0")

    def OnAnswer(self, s):
        # разбираем только факсовые ответы
        if s[:2] == '+F':
            if s == "+FCO\r":
                self.addevent( self.OnFaxConnect, None )
            # ответ если у передающего факса закончились страницы
            if s == "+FET:2\r":
                self.addevent( self.FDR, None )
            if s[:5] == '+FIS:':
                self.OnReceiveFIS(s[5:])
            # сообщение от удаленного факса, что он вешает трубку
            if s[:7] == '+FHS:00':
                self.stop_sending = True
                if self._uptubeonring:
                    self.addevent( self.at, "H0" )
            elif s[:4] == '+FHS':
                self.stop_sending = True
                if self._uptubeonring:
                    self.addevent( self.at, "H0" )

    def FDR(self):
        if self._uptubeonring:
            self.at( "+FDR", ["+FHS:00\r"] )

    def OnReceiveFIS(self,s):
        '''Процедура разбора FIS данных'''
        nums = s.strip().split(',')
        self.faxdata = FaxData()
        try:
            self.faxdata.vr = nums[0]
            self.faxdata.br = nums[1]
            self.faxdata.wd = nums[2]
            self.faxdata.ln = nums[3]
            self.faxdata.df = nums[4]
            self.faxdata.ec = nums[5]
            self.faxdata.bf = nums[6]
            self.faxdata.st = nums[7]
        except IndexError:
            pass

    def sendByte(self, ch='1'):
        self.at( ch, [], '' )

    def startReceivingFax(self, filename="test.fax" ):
        oldtimeout = self._timeout
        self._timeout = 60
        self.at( "+FDR", ["CONNECT"] )
        self._faxfilename = filename
        self._timeout = oldtimeout

    def _etxreceive( self, buf ):
        if len(self._faxfilename)>0:
            f = open( self._faxfilename, "w" )
            f.write( buf )
            f.close()
            if self._verbose:
                print "Fax received"
                sys.stdout.flush()
            self._faxfilename = ''

    def _dlereceive( self, buf ):
        if buf == 'a':
            self.modemDetected = True
            self.stop_playing = True

    def faxDial(self, number):
        self.at( "d"+str(number), ["+FCO\r"] )

    def PrepareFax( self ):
        self._timeout = 90
        self.setFaxMode()
        self.at("+FLO=2")
        self.at("+FPP=0")
        self.at("+FBO=0")
        self.at("+FNR=1,1,1,0")
        self.at("+FIE=1")
        self.at("+FCC=1,5,0,2,0,0,0,0")
        self.at("+FCR=1")
        self.at('+FLI="'+self.faxname+'"')

    def atFIS(self):
        self.at("+FIS=1,5,0,2,0,0,0,0", )

    def startSendingFax(self):
        self.bHungUp = False
        self.at( "+FDT", ["CONNECT"] )
        if self._verbose:
            print "Sending file..."
            sys.stdout.flush()

    def stopSendingFax(self):
        if self._verbose:
            print "End sending file..."
            sys.stdout.flush()
        self.at(DLE+chr(0x2e), [], NOT_AT)
#        self.at(DLE+chr(0x2e), ["+FHS:00\r"], NOT_AT)

    def nextSendingPage(self):
        if self._verbose:
            print "Next page..."
            sys.stdout.flush()
        self.at(DLE+chr(0x2c), ["OK"], "")

    def SendTiff(self, filename="test.tif"):
        oldtimeout = self._timeout
        self._timeout = 30
        self.startSendingFax()
        self._timeout = oldtimeout
        self.sendFile(os.popen2("convert " + filename + " g3:-")[1], 640 )
        self.stopSendingFax()

    def SendTiffs(self, files=[]):
        bFirst = True
        for i in files:
            if bFirst:
                bFirst = False
            else:
                self.nextSendingPage()
            self.startSendingFax()
            self.sendFile(os.popen2("convert " + i + " g3:-")[1], 640 )
        self.stopSendingFax()

    def SendG3(self, filename = "test.fax" ):
        self.startSendingFax()
        self.sendFile(open(filename), 640 )
        self.stopSendingFax()

    def SwitchFromVoiceToFax(self):
        oldtime = self._timeout
        self._timeout = 3
        try:
            self.at( "",  ["OK"] )
        except TimeoutError:
            pass
        self._timeout = oldtime
        self.at("+FCLASS=2.0", ["OK"] )
        self.at("X3D", ["OK"])

    def WaitForFax( self, timeout ):
        t = time.time()
        self.waitFaxSignal = True
        while not self.modemDetected:
            time.sleep(0.1)
            try:
                self.Run()
            except BusyAnswer:
                self.waitFaxSignal = False
                raise BusyAnswer()
            if time.time() - t > timeout:
                self.waitFaxSignal = False
                raise TimeoutError()
        self.waitFaxSignal = False

class ZyXELOmni(VoiceModem,FaxModem):
    def __init__(self, port='/dev/ttyACM0'):
        self._bVoiceDetecting = False
        self._ok = False
        try:
            VoiceModem.__init__(self, port, 57600 )
            FaxModem.__init__(self,port,57600)
        except ModemError, er:
            print er
            sys.stdout.flush()
        self._ok = True

    def DialAndSend(self, number, file=None, voice=None ):
        try:
            self._critical_write = True
            self.waitFaxSignal = False
            # если не указаны не голос ни факс дальше обрабатывать нет смысла
            if file == None and voice == None:
                return "NO VOICE NO FAX"

            self.waitFaxSignal = False
            # статус отправки по умолчанию
            self.SendStatus = "UNKNOWN ERROR"
            # устаноавливаем время ожидания команды 1,5 минуты
            self._timeout = 90
            try:
                # звоним на номер и определяем режим поднятия трубки
                self.DialAndVoiceDetect( number, 30 )
            except TimeoutError:
                return "NO ANSWER FAIL"
            except BusyAnswer:
                return "BUSY FAIL"
            ret = ""
            try:
                # если ответили голосом
                if self.detectByVoice:
                    ret+= "VOICE "
                    # если есть голосовой файл
                    if voice != None:
                        # проигрыавем его
                        self.Play( voice )
                # если ответили "пятисекундной тишиной"
                elif self.detectBySilence:
                    ret+= "SILENCE "
                    # если есть голосовой файл
                    if voice != None:
                        # проигрываем его
                        self.Play( voice )
            except BusyAnswer:
                return ret + "HUNGUP FAIL"
            # если есть факсовый файл - отправляем его
            if file != None:
                try:
                    # ждем пока на другом конце лини включат факс
                    self.WaitForFax( 20 )
                except BusyAnswer:
                    return ret+ "BUSY FAIL"
                except TimeoutError:
                    return ret + "NO FAX ANSWER FAIL"
                try:
                    # переключем модем в факс режим
                    # сложный участок соединения модем иногда умирает
                    oldtm = self._timeout
                    self._timeout = 40
                    self.SwitchFromVoiceToFax()
                    self._timeout = oldtm
                except BusyAnswer:
                    return ret+ "FCO BUSY FAIL"
                except TimeoutError:
                    self._at( "w" )
                    return ret+ "FCO TIMEOUT FAIL"
                self.SendTiff( file )
                t = time.time()
                while self.SendStatus == 'UNKNOWN ERROR':
                    time.sleep(0.2)
                    self.Run()
                    if time.time()-t > 60:
                        return ret+ "SEND FAIL"
            else:
                self.SendStatus =  "SEND OK"
            self.at("h0", [])
            self.setVoiceMode()
            return ret + self.SendStatus
        except:
            return ret + self.SendStatus

    def _etxreceive( self, buf ):
        FaxModem._etxreceive( self, buf )
        VoiceModem._etxreceive( self, buf )

    def _dlereceive( self, buf ):
        if self._verbose:
            print "<DLE>%s</DLE>" % buf
            sys.stdout.flush()
        FaxModem._dlereceive( self, buf )
        VoiceModem._dlereceive( self, buf )

    def OnAnswer(self,s):
        FaxModem.OnAnswer(self,s)
        #VoiceModem.OnAnswer(self,s)
        if s[:7] == '+FHS:00':
            self.SendStatus = "SEND OK"
        elif s[:4] == '+FHS':
            self.SendStatus = "SEND FAIL "+s[:-1]

    def DialAndVoiceDetect(self, number, timeout=15 ):
        self.setRingbackCame(0)
        self.TakeUp()
        self.dial(number, timeout)
        self.detectVoiceAnswer(timeout)

    def Busy(self):
        if self._bVoiceDetecting == True:
            self._bVoiceDetecting = False
            raise BusyAnswer()
        if self.waitFaxSignal:
            raise BusyAnswer()
        self.stop_playing = True
        #self.addevent( self.HungUp, None )

    def detectVoiceAnswer(self, timeout=-1):
        if timeout == -1:
            timeout = self._timeout
        self.modemDetected = False
        self.detectByVoice = False
        self.detectBySilence = False
        self.startRecordVoice("gudki.rmd")
        intervalCounter = 1
        silenceCounter = 0
        try:
            c = time.time()
            self._bVoiceDetecting = True
            while self._bVoiceDetecting and not self.modemDetected:
                if len(self._buf) >= 2400* intervalCounter:
                    k = os.popen2( \
                       "rmdtopvf | pvftowav | /usr/local/bin/" + \
                       "voicedetector %d %d %d %d %d" \
                       % ((intervalCounter-1)*4800, 9600, 15000, 1300, 600 ))
                    k[0].write("RMD1ZyXEL Omni 56K" + NUL * 3 + chr(4) \
                            + chr(0x25)+chr(0x80)+chr(0x04)+NUL*7 )
                    k[0].write( self._buf )
                    k[0].flush()
                    k[0].close()
                    otvet = k[1].read()[0]
                    if self._verbose:
                        print otvet
                        sys.stdout.flush()
                    if otvet == '2':
                         self.detectByVoice = True
                         if self._verbose:
                             print "Voice answer"
                             sys.stdout.flush()
                         break;
                    elif otvet == '1':
                         silenceCounter = 0
                    else:
                        silenceCounter+= 1
                        if silenceCounter > 12:
                            self.detectBySilence = True
                            break;
                    k[1].close()
                    intervalCounter+= 1
                else:
                    if time.time()-c > timeout:
                        self.stopRecordVoice()
                        raise TimeoutError()
                    time.sleep(0.1)
                    self.Run()
        except BusyAnswer:
            self.stopRecordVoice()
            raise BusyAnswer()
        except KeyboardInterrupt:
            pass
        self.stopRecordVoice()

    def HardInit(self, verbose=False):
        '''Процедура полного сброса модема.
        
        Если проигрывание файла прошло со сбоями или при связи произошли
        какие либо проблемы, то при последующих проигрываниях звука будут
        шипения или он может слишком быстро проигрывться. Действия в этой
        процедуре позволяют перевести модем в нормальный режим.
        '''
        self._critical_write = False
        self.Init(verbose)
        self._timeout =2
        try:
            self.TakeUp()
            self.Play( "welcome.rmd" )
        except:
            pass
        try:
            self.stopPlayVoice()
        except:
            pass
        try:
            self.HungUp()
        except:
            pass
        self._timeout = 30
        self.reset()
        self.reset()
        self.Init(verbose)
        self.PrepareFax()
        self.at("X3D",[])
        time.sleep(1)
        self.at("",["NO CARRIER"])
        self.setVoiceMode()

    def Dial(self, number):
        self.TakeUp()
        self.dial( number )

    def TakeUp(self):
        k = self.getCurMode()
        if len(k)>0 and k[0] != '8':
            self.setVoiceMode()
        self.setVoiceInputOutput(2)

    def HungUp(self):
        k = self.getCurMode()
        if len(k)>0 and k[0] != '8':
            self.setVoiceMode()
        self.setVoiceInputOutput(0)

    def Init(self, verbose=False):
        '''Инициализация модема  для работы в голосовом режиме
         verbose режим, когда отображается ход процесса'''
        self._verbose = verbose
        self.test()
        self.reset()
        self.setDataMode()
        self.setVoiceMode()
        self.setVoiceInputOutput(0)
        self.setInactiveTime(100)
        self.setSilenceDetection(12,70)
        self.setTransmitGain(128)
        self.setReceiveGain(128)
        self.at( "X4S40.3=1S40.4=1S40.5=1S40.6=1" )
        self.setRingbackAway(70)
        self.setRingbackCame(100)

    def Play(self, name ):
        self.startPlayVoice()
        if name[-4:] == '.wav':
            if self._verbose:
                print "Playing wav"
            self.playWavFile(name)
        else:
            if self._verbose:
                print "Playing rmd"
            self.playFile(name)
        self.stopPlayVoice()

    def Close(self):
        if self._ok:
            self._ok = False
            self.close()

def DialOnList( m, listfile ):
    f = open( listfile, "r" )
    strs = f.readlines()
    f.close()

def Dozvon( m, num ):
    f = False
    while not f:
        try:
             m.DialAndVoiceDetect(num,20)
             f = True
        except BusyAnswer:
             print "BUSY redialing after 5 sec"
             sys.stdout.flush()
             m.HungUp()
             time.sleep(5)
        except TimeoutError:
             print "Nobody in place redialing 10 sec"
             sys.stdout.flush()
             m.HungUp()
             time.sleep(5)
    m.Play("welcome.rmd")
    m.HungUp()



