#!/usr/bin/python
# -*- coding: utf-8 -*-
#Copyright 2008 Calculate Pack, http://www.calculate-linux.ru
# 
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
# 
#      http://www.apache.org/licenses/LICENSE-2.0
# 
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

import sys
import serial
import threading
import advmodem
import time
import datetime
import string
import os
import serial
import getopt
import re
from signal import SIGTERM, signal, SIGUSR1, SIGKILL

import random

def usage():
    print '''
Использование: faxer.py --start|--stop [options]

 --start                  Запустить/перезапустить службу
 --stop                   Остановить службу

Опции:
 --pidpath              Директорий, куда будет писаться pid файл.
                        По умолчанию пишет в /var/run
 --logpath              Директорий, куда будет писаться log демона
                        По умолчанию пишет в /var/log
 --errpath              Файл в который будет выведен stderr
                        По умолчанию пишет в log
 --devmodem             порт к которому подключен модем
                        По умолчанию /dev/ttyACM0
 --filenumber           Файл с номерами на которые будет отправлен факс
                        По умолчанию telnumber
                               
                        Формат файла:
                        номер[tab]факсовый tif[tab]голосовой файл
                        Если использовать ключевое слово None вместо имени
                        файла, то факс или сообщение отправлены не будут

 --defaultfax           Файл факса для всех номеров, кроме тек у которых
                        конкретно указан факс-файл.
                        По умолчанию 'None' 
 --defaultvoice         Файл голосового приветствия для всех номеров.
                        По умолчанию 'None'
 --busytimerecall       Время в секундах, через которое будет повторный
                        звонок, если при попытке дозвонится было занято.
                        По умолчанию 5 минут.
 --failtimerecall       Время в секундах, через которое будет повторный
                        звонок, если при дозовоне либо не взяли трубку,
                        либо передача по каким-то причинам не состоялась.
                        По умполчанию 1 час.
 --trysend              Количество неудачных попыток после которых звонки
                        на этот номер прекращаются. Занято считается за 1/3
                        попытки.
 '''


def GetCurrentTime():
    '''Получить текущее время в класс datetime
    '''
    t = time.localtime()
    return datetime.datetime( t[0],t[1],t[2],t[3],t[4],t[5],t[6] )

class Daemon:
    '''Класс реализующий функционал демона работающего на pid'''
    def __init__(self, name, **kwargs):
        '''Инициализация демона.
        kwargs[errpath] путь к файлу для stderr, None по умолч
        kwargs[inpath] путь к файлу для stdin, /dev/null
        kwargs[pidpath] путь к pid файлу (локальный коталог)
        kwargs[logpath] путь для stdout
        '''
        self._name = name
        self._events = []

        if kwargs.has_key('errpath') and kwargs['errpath'] != None:
            self._erpath = kwargs['errpath']
        else:
            self._erpath = None

        if kwargs.has_key('inpath') and kwargs['inpath'] != None:
            self._inpath = kwargs['inpath']
        else:
            self._inpath = '/dev/null'

        if kwargs.has_key('pidpath') and kwargs['pidpath'] != None:
            self._path = kwargs['pidpath']
        else:
            self._path = os.path.realpath(os.path.dirname(sys.argv[0]))

        if kwargs.has_key('logpath') and kwargs['logpath'] != None:
            self._log = kwargs['logpath']
        else:
            self._log = os.path.realpath(os.path.dirname(sys.argv[0]))

        self._path+= "/" + name + ".pid"
        self._log+= "/" + name + ".log"

    def Initialize(self, **kwargs ):
	'''
           в ходе демонизации меняется pid, то некоторые установки необходимо
           делать после - это именно тот метод.
        '''
        pass

    def AddTask(self, fn, intervalsec):
        '''Добавить задачу на выполение.
        Параметры:
            fn - функция для выполнения (НЕ ЦИКЛИЧЕСКАЯ!!!),
            intervalsec - интервал вызовов в секунду'''
        self._events.append( [fn, intervalsec, None] )

    def _deamonize(self,stdout = '/dev/null', stderr = None, \
                  stdin ='/dev/null', \
                  pidfile = None, startmsg = 'started with pid %s' ):
        '''Внутренняя функция демонизации.
        Если параметры stdout, stdin не переданы функции, то они утанавливаются
        как /dev/null, stderr (ошибки) не выводятся'''
        # Делаем первый fork.
        try:
            pid = os.fork()
            if (pid > 0):
                sys.exit(0)# Выходим из первого родителя.
        except OSError, e:
            # В случае ошибки сообщаем об этом
            sys.stderr.write("fork #1 failed: (%d) %s\n" % (e.errno, \
                                                            e.strerror))
            sys.exit(1)

        # Отделяемся от родительского окружения сменяя директорию.
#        os.chdir(myvar.daemon_place)
        os.umask(0)
        os.setsid()

        # Делаем второй fork.
        try:
            pid = os.fork()
            if (pid > 0):
                sys.exit(0) # Выходим из второго родителя.
        except OSError, e:
        # Отрабатываем возможные ошибки
            sys.stderr.write("fork #2 failed: (%d) %s\n" % (e.errno, \
                                                            e.strerror))
            sys.exit(1)

        # Открываем дескриптор файла и пишем стартовое сообщение
        if (not stderr):
            stderr = stdout

        si = file(stdin, 'r')
        so = file(stdout, 'a+')
        se = file(stderr, 'a+', 0)
        self._pid = str(os.getpid())
        sys.stderr.write("\n%s\n" % startmsg % self._pid)
        sys.stderr.flush()
        if pidfile:
            file(pidfile,'w+').write("%s\n" % self._pid)

        # Переопределяем стандартные файловые дескрипторы.
        os.dup2(si.fileno(), sys.stdin.fileno())
        os.dup2(so.fileno(), sys.stdout.fileno())
        os.dup2(se.fileno(), sys.stderr.fileno())
        self._pid = int(self._pid)

    def _getterStopSignal(self, signal, framestack ):
        '''Внутренняя функция получения пользовательского сигнала завершения
           (SIG_USR1), сбрасывает флаг продолжения работы демона, после чего
           демон после завершения всех задач корректно заврешает свою работу.
           Желательно в случае замещения обеспечить сброс флага работы.
        '''
        self._workflag = False


    def Run(self, **kwargs):
        '''Главный цикл работы демона, в нем происходит вызов по времени
           добавленных задач и проверка завершения.'''
        # демонизируем поток
        self._deamonize( stdout = self._log, stderr = self._erpath, \
                         stdin=self._inpath, pidfile=self._path )
        # перехватываем сигнал терм
        signal( SIGTERM, self._getterStopSignal )
        # взводим флаг циклической работы
        self._workflag = True
	# дополнительная инициализация
        self.Initialize(**kwargs)
	if self._workflag:
	    self.WriteLog( "Daemon %s(%d) has started" % \
                           (self._name, self._pid) )

        # цикл до сигнала завершения или pid завершения
        while(self._workflag):
            # читаем pid из файла и переводим в число
            # работа завершается если pid файл отсутствует, pid номера
            # разные, в pid содержится не числовая информация
            try:
                readpid = int(open( self._path ).read());
                if self._pid != readpid:
                    break
            except IOError:
                break
            except ValueError:
                break

            # получаем текущее время
            curtime = GetCurrentTime()
            # перебираем список задач демона
            for i in self._events:
                # если пора вызывать задачу
                if (i[2] == None or (curtime-i[2]).seconds > i[1]):
                    # вызываем ее и перехватываем все некорректные вызовы
                    try:
			i[0]()
                    except:
                        self.WriteLog( "%s некорректно отработала" %\
                            i[0].func_name )
                    # устанавливаем текущее время
                    i[2] = curtime
            # засыпаем на секуду
            time.sleep( 0.1 )
        # если выход из цикла по флагу (завершение, через подачу сигнала)
        # удаляем pid файл
        if self._workflag == False:
            try:
                os.remove(self._path)
            except OSError:
                pass
        self.WriteLog( "Daemon %s(%d) is terminated" % \
                           (self._name, self._pid) )
	self.Destructor()

    def Destructor(self):
	'''Действия, которые необходимо выполнить перед завершение демона'''
	pass

    def Terminate(self):
        '''Завершение демона 
	
	Вернее инициатор завершения демоан.Основывается на правильно указанном
	pid файле, в противном случае дает не правильный результат.
        '''
        # читаем pid файл
        try:
            pid = open( self._path ).read()
        except IOError,er:
            print "Демон не запущен"
            return

        try:
            pid = int(pid)
            try:
                # отправляем сигнал SIGTERM процессу для корректного завершения
                # его работы
                os.kill( pid, SIGTERM )
                # 10 секунд на корректное завершение демона
                for i in xrange( 0, 10 ):
                    # открываем pid файл, если демон завершится, то
                    # исключение ввода вывода сигнал корректного завершения
                    open( self._path ).close()
                    # спим секунду
                    time.sleep( 1 )
                # если демон не завершился за 10 секунд убиваем его
                os.kill( pid, SIGKILL )
                os.remove( self._path )
            # корректный выход
            except IOError:
                pass
            # проблемы с удалением
            except OSError:
                pass
        except ValueError, er:
            print "Pid файл не корректный"

    def WriteLog(self, s):
        '''Записать строку в лог файл в формете:
        [локальное время] строка сообщения
        '''
        try:
            print "[%s]\t%s" % (time.strftime("%d/%m/%Y %H:%M:%S", \
                                                   time.localtime() ), s )
            sys.stdout.flush()
        except:
            pass

class PhoneInformation:
    def __init__(self, number, trycount=10, busyredial=2, \
	         noanswerredial=4):
	'''Инициализация объекта
	
	Входные параметры:
	    number - номер на который звонить
	    trycount - количество попыток на дозвон (5)

	'''
	self.number = number
	self.welcome = 'welcome.rmd'
	self.faxfile = None
	self.trycount = int(trycount)
	self.sendfail = 0
	self.busy = 0
	self.noanswer = 0
	self.lasttime = None
	self.busyredial = int(busyredial)
	self.noanswerredial = int(noanswerredial)
	self.statusOK = False

    def SetFax(self, faxfile):
	'''Установить факсимильное сообщение связанное с этим номером
	
	Параметры:
	    faxfile имя факсового файла (tif) если None, то факс не отсылается
	'''
	self.faxfile = faxfile
	if faxfile == 'None':
	    self.faxfile = None
	elif not os.path.isfile( faxfile ):
	    self.trycount = 0
	    raise OSError, "факсимильный файл не найден"

    def GetFax(self):
	'''Получить факсимильное сообщение связанное с этим номером'''
	return self.faxfile

    def SetOK(self, attime):
	'''Сообщить объекту - успешная отправка
	
	Параметры:
	    attime - время события
	'''
	self.lasttime = attime
	self.statusOK = True

    def SetWelcome(self, voicefile):
	'''Установить приветственное сообщение связанное с этим номером
	
	Параметры:
	    voicefile имя звукового файла (wav/rmd) если None, то без голоса
	'''
	self.welcome = voicefile
	if voicefile == 'None':
	    self.welcome = None
	elif not os.path.isfile(voicefile):
	    self.trycount = 0
	    raise OSError, "голосовой файл не найден"

    def GetWelcome(self):
	'''Получить приветственное сообщение связанное с этим номером'''
	return self.welcome

    def AddBusy(self, attime):
	'''Сообщить объекту - занято 
	
	Параметры:
	    attime - время события
	'''
	self.busy += 1
	if self.busy > 2:
	    self.AddNoAnswer(attime)
	self.lasttime = attime

    def AddNoAnswer(self, attime):
	'''Сообщить объекту - нет ответа 
	
	Параметры:
	    attime - время события
	'''
	self.noanswer += 1
	self.busy = 0
	self.lasttime = attime

    def AddSendFail(self, attime):
	'''Сообщить объекту - не удалось отправить
	
	Параметры:
	    attime - время события
	'''
	self.sendfail += 1
	self.busy = 0
	self.lasttime = attime

    def IsReady(self ):
	'''Пора ли отправлять факс на данный номер'''
	if self.statusOK:
	    return False
	elif self.lasttime == None:
	    return True
	elif self.busy > 0:
	    if time.time() - self.lasttime > self.busyredial:
		return True
	    else:
		return False
        elif time.time() - self.lasttime > self.noanswerredial:
	    return True
	else:
	    return False

    def IsFail(self):
	'''Узнать стоит ли отправлять на данный номер факс повторно
	
	Количество попыток закончилось
	'''
	if self.noanswer + self.sendfail >= self.trycount:
	    return True
	else:
	    return False

class ZyXELOmniFake:
    '''Класс заглушка для тестирования демона'''
    def __init__( self, **kwargs ):
	pass

    def HardInit(self, verbose):
	pass

    def reset(self):
	pass

    def close(self):
	pass

    def Close(self):
	pass

    def PrepareFax(self):
	pass

    def setVoiceMode(self):
	pass

    def DialAndSend(self, number, fax, voice):
	listResult= ("SEND OK", "BUSY FAIL", "FCO FAIL", "UNKNOWN ERROR", \
			"VOICE FCO FAIL", "BUSY FAIL", "FCO FAIL", "UNKNOWN ERROR", \
			"VOICE FCO FAIL", "BUSY FAIL", "FCO FAIL", "UNKNOWN ERROR", \
			"VOICE FCO FAIL", "BUSY FAIL", "FCO FAIL", "UNKNOWN ERROR", \
			"VOICE FCO FAIL")
	if fax != None and voice != None:
	    print "Send on number "+str(number)+fax, "and", voice
	elif fax == None and voice == None:
	    print "Bad dialing"
	elif fax == None:
	    print "Voice without fax"
	else:
	    print "Fax without voice"
	return listResult[ random.randint(0, len(listResult)) ]


class FaxerDaemon(Daemon):
    '''Демон для отправки факсов

    Для запуска используется Run.
    При инициализации обязательные параметры
        kwargs[errpath] путь к файлу для stderr, None по умолч
        kwargs[inpath] путь к файлу для stdin, /dev/null
        kwargs[pidpath] путь к pid файлу (локальный коталог)
        kwargs[logpath] путь для stdout
	kwargs[filenumber] имя файла с номерами
    '''
    def __init__(self, name, **kwargs):
	Daemon.__init__(self, name, **kwargs )
	self.AddTask( self.OneTask, 0 )
	# указываем персональный лог для файла с номерами
	self._log = self._log[:-3] + kwargs['filenumber'].replace('/','_') + '.log'

    def Initialize(self, **kwargs):
	'''Главная инициализация демона

	Обязательные параметры kwargs:
	    verbose - True/False дополнительная отладочная информация в лог
	    faxdefault - факс-файл по умолчанию
	    voicedefault - голосовой файл по умолчанию
	    filenumber - файл с номерами телефонов
	    trysend - количество попыток отправить на данный номер
	    busytimerecall - время в секундах на перезвон на этот номер
	    failtimerecall - время в секундах на перезвон при провале
	'''
	# чтение номеров из файла номеров
	self.listForDel = []
	self.listOK = []
	self.listFAIL = []
	self.nums = {}
	self.verbose = kwargs['verbose']
	self.defaultFax = kwargs['faxdefault']
	self.defaultVoice = kwargs['voicedefault']
	# читаем построчно файл с номерами телефонов
	# номер факсовыйФайл голосовойФайл
	for i in open( kwargs['filenumber'] ):
	    parts = i.split()
	    # создаем запись о телефоне
	    self.nums[ parts[0] ] = PhoneInformation(parts[0], \
				kwargs['trysend'], kwargs['busytimerecall'],\
				kwargs['failtimerecall'])
	    try:
		# если указан факс
		if len(parts) > 1:
		    # устанавливаем его в запись о телефоне
		    self.nums[ parts[0] ].SetFax( parts[1] )
		    # если указан голос
		    if len(parts) > 2:
			# устанавливаем его в запись о телефоне
			self.nums[ parts[0] ].SetWelcome( parts[2] )
		    else:
			# устанавливаем звук по умолчанию
			self.nums[ parts[0] ].SetWelcome( self.defaultVoice )
		else:
		    # устанавливаем звук и факс по умолчанию
		    self.nums[ parts[0] ].SetFax( self.defaultFax )
		    self.nums[ parts[0] ].SetWelcome( self.defaultVoice )
	    except OSError, er:
		self.WriteLog( "Для номера "+str(parts[0])+ " " + str(er) )
	# реинициализация номеров
	reg = re.compile( "(\[[^\]]+\])\s+\*([\w,.]+)\s+(.+)$", re.S )
	# читаем лог
	for strOfLog in open( self._log ):
	    try:
		# каждую строку обрабатываем по статусу
		findobj = reg.findall(strOfLog)
		if len(findobj) == 0 or \
			len(findobj[0]) < 3:
		    continue
		gettime, num,status = findobj[0]
		try:
		    gettime = time.mktime(time.strptime(gettime, "[%d/%m/%Y %H:%M:%S]"))
		except:
		    gettime = None
		self.ParseStatus(num, status.strip(), gettime)
	    # намеренное использование общего эксепта
	    except:
		pass
	# если список еще не пуст
	for i in self.listForDel:
	    try:
		self.listOK.append(self.nums.pop(i))
	    except KeyError:
		pass
	self.k = iter(self.nums)
#	self.modem = ZyXELOmniFake()
	self.modem = advmodem.ZyXELOmni()
	# пытаемся не более трех раз проинициализировать модем, в случае
	# провала всех трех попыток сбрасываем _workflag работа демон 
	# не имеет смысла
	for i in range(1,3):
	    try:
		self.modem.HardInit(self.verbose)
		return;
	    except:
		pass
	self._workflag = False
	self.WriteLog( "Ошибка получения доступа к модему" )

    def ParseStatus(self, number, status, attime=-1):
	'''Разбирает статус передачи сообщения.
	
	Параметры:
	    number номер телефона для звонка
	    status строка статус полученная после отправления факса
	Возвращаемые значения:
	    True если рекомендуется выполнить реинициализацю модема
	    False если реинициализация не требуется
	'''
	if attime == -1:
	    attime = time.time()
	if status.find('SEND OK') >= 0:
	    self.nums[number].SetOK(attime)
	    self.listForDel.append( number )
	    return False
	elif status == 'BUSY FAIL':
	    self.nums[number].AddBusy(attime)
	    return False
	elif status.find('FCO FAIL') >= 0:
	    self.nums[number].AddSendFail(attime)
	    return True
	else:
	    self.nums[number].AddNoAnswer(attime)
	    return True

    def OneTask(self):
	'''Одна из задач

	Достает из списка следующий номер и отправляет на него факс. С отмечанием в логе.
	'''
	try:
	    # флаг, что в течении проходна не был обработан телефон
	    bPass = True
	    # проверяем не пора ли гасить поток или звонок по номеру произведен
	    while self._workflag and bPass:
		# получаем следующий номер из списка
		num = self.k.next()
		# если на номер больше не надо звонить
		if self.nums.has_key(num) and self.nums[num].IsFail():
		    # выбрасываем его из списка
		    self.listForDel.append( num )
		    # отпиываем в лог сообщение что номер выброшен
		    self.WriteLog("%s\t%s" % (num, "DISCARD"))
		    continue
		# если на него пора звонить
		if self.nums[num].IsReady():
		    # звоним на него
		    try:
			ret = self.modem.DialAndSend( num, self.nums[num].GetFax(), \
					      self.nums[num].GetWelcome() )
		    except:
			ret = "UNKNOWN ERROR"
		    # разбираем статус отправки и при необходимости
		    if self.ParseStatus( num, ret ):
			# реинициализируем модем
			for i in range(1,3):
			    try:
				self.modem.HardInit(self.verbose)
				break
			    except:
				pass
		    # отписываем в лог номер и статус отправки
		    # перед номером добавляется *, для легкого различия
		    # статуса дозвона и прочих сообщений демона
		    self.WriteLog("*%s\t%s" % (num, ret))
		    # отмечаемся что на номер произведен звонок
		    bPass = False
	# в случае если прошли весь список
    	except StopIteration:
	    # если список еще не пуст
	    for i in self.listForDel:
		try:
		    numinfo = self.nums.pop(i)
		    if numinfo.IsFail():
			self.listFAIL.append(numinfo)
		    else:
			self.listOK.append(numinfo)
		except KeyError:
		    pass
	    if len( self.nums) > 0:
		# запускаем итератор повторно сначала
		self.k = iter( self.nums )
	    else:
		# иначе сообщаем демону, что надо завершить работу
		self.WriteLog( "All done" )
		print "-"*20
		print "Sending OK(%d):\n" % len(self.listOK) + "-"*20
		for i in self.listOK:
		    print i.number,"\t",time.ctime(i.lasttime)
		print "-"*20
		print "Sending FAIL(%d):\n" % len(self.listFAIL)+ "-"*20
		for i in self.listFAIL:
		    print i.number,"\t",time.ctime(i.lasttime)
		print "-"*20

		self._workflag = False

    def Destructor(self):
	'''Сбрасываем модем, закрываем порт'''
	self.modem.reset()
	self.modem.Close()


def main(argv=sys.argv):
    try:
	opts, args = getopt.getopt(sys.argv[1:], "d:f:", ["pidpath=", \
            "errpath=", "logpath=", "inpath=", "start", "stop", \
	    "devmodem=", "filenumber=", "defaultfax=", "defaultvoice=", \
	    "trysend=", "busytimerecall=", "failtimerecall=",\
	    "verbose"] )
    except getopt.GetoptError, err:
	print str(err)
	usage()
	sys.exit(2)

    cmd = ''
    pidpath = '/var/run'
    errpath = None  #'/dev/null'
    logpath = '/var/log'
    inpath = '/dev/null'
    devmodem = '/dev/ttyACM0'
    filenumber = 'telnumbers'
    faxdefault = 'None'
    voicedefault = 'None'
    trysend = 7
    busytimerecall = 300
    failtimerecall = 3600
    verbose = False

    for o, a in opts:
	if o == "--defaultfax":
	    faxdefault = a
	elif o == "--verbose":
	    verbose = bool(a)
	elif o == '--busytimerecall':
	    busytimerecall = int(a)
	elif o == '--failtimerecall':
	    failtimerecall = int(a)
	elif o == '--defaultvoice':
	    voicedefault = a
	elif o == '--trysend':
	    trysend = int(a)
	elif o == "--start":
	    cmd = 'start'
	elif o == '--stop':
	    cmd = 'stop'
	elif o == '--pidpath':
	    pidpath = a
	elif o == '--errpath':
	    errpath = a
	elif o == '--logpath':
	    logpath = a
	elif o == '--inpath':
	    inpath = a
	elif o in ('--devmodem', '-d'):
	    devmodem = a
	elif o in ('-f', '--filenumber'):
	    filenumber = a

    if cmd == 'start':
	    d = FaxerDaemon("faxdaemon", pidpath=pidpath, \
		logpath=logpath, inpath=inpath, errpath=errpath, \
		filenumber=filenumber) \
		.Run( devmodem=devmodem, filenumber=filenumber, \
		      faxdefault=faxdefault, voicedefault=voicedefault, \
		      trysend=trysend, busytimerecall=busytimerecall, \
		      failtimerecall=failtimerecall, verbose=verbose )
    elif cmd == 'stop':
	FaxerDaemon( "faxdaemon", pidpath=pidpath, \
		logpath=logpath, inpath=inpath, errpath=errpath, \
		filenumber=filenumber).Terminate()
    else:
	usage()

if __name__ == '__main__':
     main()
     sys.exit(1)
