//  Copyright 2008 Calculate Pack, http://www.calculate-linux.ru
//  Licensed under the Apache License, Version 2.0 (the "License");
//  you may not use this file except in compliance with the License.
//  You may obtain a copy of the License at
// 
//      http://www.apache.org/licenses/LICENSE-2.0
// 
//  Unless required by applicable law or agreed to in writing, software
//  distributed under the License is distributed on an "AS IS" BASIS,
//  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//  See the License for the specific language governing permissions and
//  limitations under the License.

#include <stdio.h>
#include <stdlib.h>
#include <fftw3.h>

static double *rdata = NULL, *idata = NULL;
static fftw_plan rplan, iplan;
static int last_fft_size = 0;

double A[19200];
int n = 9600;
int i,m,k;
char erg[35];
char erg2[35];
double absval;
int is16bitfile = 0;
char inc[19200];

void usage()
{
    printf( "voicedetector [offset] [size] [toneamp] [voiceamp] [voicecount]\n\
    Анализирует wav-файл поступающий из stdin и возвращает:\n\
    0 - если тишина\n\
    1 - если тоновый сигнал\n\
    2 - если спектр похож на голос\n\
    \n\
    Опции:\n\
    offset     -- смещение от начала файла в байтах\n\
    size       -- размер анализируемой части файла в байтах\n\
    toneamp    -- минимальная амплитуда тонового сигнала\n\
    voiceamp   -- минимальная амплитуда голосового сигнала\n\
    voicecount -- минимальное количество спектральных частот превышающих voiceamp\n\
" );
}

int main(int argc, char *argv[]) {
	int retcode= 0;
	double cc;
	int v, val, val2;
	int samplerate = 9600;
	int bandwidth = samplerate / 2;
	double tonamp= 15000;
	double voiceamp= 2000;
	int countVoice= 400;
	int passN= 0;
	// получаем параметры
	// 1 - сколько байт от файла необходимо пропустить
	// 2 - сколько необходимо прочитать
	// 3 - амплитуда тонового сигнала
	// 4 - амплитуда голосового сигнала
	// 5 - сколько частот должно превысить амплитуду голосового сигнала
	//     так что можно считать что это голос
	if(argc>1)
	{
	   if(strcmp( argv[1], "--help")==0)
	   {
	       usage();
	       exit(1);
	   }
	   passN= atoi(argv[1]);
	   if(argc>2)
	   {
	       n= atoi( argv[2] );
	       if(argc>3)
	       {
	           tonamp= atof( argv[3] );
	           if(argc>4)
	           {
	                voiceamp= atof( argv[4] );
	                if(argc>5)
	                {
	                      countVoice= atoi( argv[5] );
	                }
	           }
	       }
	   }
	}
	int nhailf = n / 2;
	FILE *file;
	double correction = (double)samplerate / (double)n;
	int count2000= 0;

	// skip wav file header
	fgets(inc,37,stdin);
	fgets(inc,9,stdin);
        
//	file = fopen("data.dat", "w");
	// read data and fill "A"-array
	memset( (void*)A, 0, n*sizeof(char) );
	for (v=-passN;v<n;v++) {
	       if( v >= 0 )
		A[v] = getc(stdin);
	       else
	           getc(stdin);
	}
	// prepare fft with fftw
	rdata = (double *)fftw_malloc(n * sizeof(double));
	idata = (double *)fftw_malloc(n * sizeof(double));
	// create the fftw plan
	rplan = fftw_plan_r2r_1d(n, rdata, idata, FFTW_R2HC, FFTW_FORWARD); 

	// we have no imaginary data, so clear idata
	memset((void *)idata, 0, n * sizeof(double));
	// fill rdata with actual data
	for (i = 0; i < n; i++) { rdata[i] = A[i]; }
	// make fft
	fftw_execute(rplan);
	
	// post-process FFT data: make absolute values, and calculate
	//   real frequency of each power line in the spectrum
	m = 0;
	for (i = 0; i < (n-2); i++) {
		absval = sqrt(idata[i] * idata[i]);
		cc = (double)m * correction;
		if( i > 50 )
		{
		    if( cc >= 420 && cc <= 427 && absval > tonamp)
		    {
		       retcode= 1;
		    }
		    if( absval > voiceamp )
		    {
			count2000++;
		    }
		}
		sprintf(erg, "%f %f\n", cc, absval);
//		fputs(erg, file);
		m++;
	}

	// housekeeping
	// если количество аплитуд выше голосоывх больше допустимого - голос
	if( count2000 > countVoice)
	{
	    retcode= 2;
	}
//	fclose(file);
	fftw_destroy_plan(rplan);
	fftw_free(rdata);
	fftw_free(idata);
	printf("%d\n", retcode);
	printf("Voice amplitue=%d\n", count2000);
	return retcode;
} 
